function subs = backwardsSelection(df, minSize)
%function subs = subsetSelection(df, maxSize, type)
%
%Do subset selection on df, using the forward selection heuristic, looking
%at subsets of increasing size. 
%Returns a struct array of size 1..maxSize, containing selected features
%vector (colidx), rmse and mae of the model (on training data)

error(nargchk(2,2,nargin));

num_train_instances = size(df.traindata.features,1);
num_test_instances = size(df.validdata.features,1);
num_features = size(df.traindata.features,2);
minSize = max(minSize,1);
Xtest = [ones(num_test_instances,1) df.validdata.features];
features = [1:num_features];

display (sprintf('Backwards selection started: %d features total; smallest model is %d',num_features,minSize));

X = [ones(size(df.traindata.features,1),1) df.traindata.features];
%global delta;
delta = 1e-3;
A = X'*X+delta*eye(num_features+1,num_features+1);
Ainv = inv(A);
XtY = X' * df.traindata.response;
compute_training_RMSE = false;

%test = feval(df.backtransform, df.validdata.response, df);
test = df.validdata.response;

% do the full model
model = Ainv * XtY;
preds = Xtest * model;
%preds = feval(df.backtransform, preds, df);
subs(num_features).MAE = mean(abs(preds-test));
subs(num_features).RMSE = sqrt(mean( (preds-test).^2 ));
if (compute_training_RMSE)
	preds = [X df.traindata.features(:,tt)]*model;
	%preds = feval(df.backtransform, preds, df);
	%subs(num_features).trainingRMSE = sqrt(mean((preds-feval(df.backtransform, df.traindata.response, df)).^2));
	subs(num_features).trainingRMSE = sqrt(mean((preds-df.traindata.response).^2));
end
subs(num_features).features = features;
subs(num_features).method = 'backwardsSelection';

% for every set size
for t=num_features-1:-1:minSize
	
	minRMSE = inf;
	best_subset = 0;

	if (mod(t,5)==0)
		display (sprintf('  backwards selection: subset size %d', t));
	end
	
	% for each feature in the set
	for tt=1:size(df.traindata.features,2) %- size(features,2)
		if (isempty(find(features==tt)))
			continue;
		else
			k = find(features==tt);
		end
		
		%features(t)=tt;
	
		% construct the model with this feature removed
		model = modelRemoveFeature(X, k, df.traindata.response, A, Ainv, delta, XtY);
		
		if (model == -1)
			display(sprintf('    Prevented division by zero in backwardsSelection - skipped deletion of feature %d',tt));
			continue;
			
			% modelRemoveFeature divided by zero: this feature needs to go
			best_subset = tt;
			result(tt).features = features(1:end~=find(features==tt));
			result(tt).method = 'backwardsSelection';
			% minRMSE = min(minRMSE, result(tt).RMSE);  -- Leave minRMSE as it is
			break;
		end
		
		% record the RMSE and MAE on validation data for this feature
		%Xtest(:,t+1) = df.validdata.features(:,tt);
		preds = Xtest(:,1:end~=k) * model;
		
		% are we sure we want to do this?  This means we're doing model
		% selection based on a different error measure from the one we
		% optimize in the regression...
		%preds = feval(df.backtransform, preds, df);

		result(tt).MAE = mean(abs(preds-test));
		result(tt).RMSE = sqrt(mean( (preds-test).^2 ));

		if (compute_training_RMSE)
			preds = X(:,1:end~=k)*model;
			%preds = feval(df.backtransform, preds, df);
			%result(tt).trainingRMSE = sqrt(mean((preds-feval(df.backtransform, df.traindata.response, df)).^2));
			result(tt).trainingRMSE = sqrt(mean((preds-df.traindata.response).^2));
		end
		
		result(tt).features = features(1:end~=find(features==tt));
		result(tt).method = 'backwardsSelection';
		minRMSE = min(minRMSE, result(tt).RMSE);
		if (minRMSE == result(tt).RMSE)
			best_subset = tt;
		end
	end

	% pick the feature with the best RMSE
	subs(t) = result(best_subset);
	clear result;
	k = find(features == best_subset);
	features = subs(t).features;
	
	% update the inverse; global variables
	[model A Ainv XtY] = modelRemoveFeature(X, k, df.traindata.response, A, Ainv, delta, XtY);
	X = X(:,1:end~=k);
	Xtest = Xtest(:,1:end~=k);
end

display 'Backwards selection finished';