function subs = exhaustiveSelection(df, maxSize)
%function subs = exhaustiveSelection(df, maxSize, type)
%
%Do exhaustive subset selection on df, looking
%at subsets of increasing size. 
%Returns a struct array of size 1..maxSize, containing selected features
%vector (colidx), rmse and mae of the model (on training data)

error(nargchk(2,2,nargin));

display (sprintf('Exhaustive selection started: %d features total',size(df.traindata.features,2)))

num_features = size(df.traindata.features,2);
maxIteration = min(maxSize,num_features);

for t=1:maxIteration
    
    %all possible subsets of this size
    candidates=nchoosek(1:num_features, t);    
    nSubsets=size(t,1);
    
    bestRMSE=Inf;
    
    if (mod(t,5)==0)
		display (sprintf('  exhaustive selection: subset size %d of %d', [t maxIteration]))
	end
    
    for tt=1:nSubsets
        %compute model
        stats = modelStats(projectDF(df, candidates(tt,:)), true, false);
        
        if (stats.RMSE < bestRMSE) 
            bestRMSE = stats.RMSE;
            subs(t).MAE = stats.MAE;
            subs(t).RMSE = stats.RMSE;
            subs(t).features = candidates(tt,:);
            subs(t).method = 'exhaustive';
        end
        
    end
    
end

display 'Exhaustive selection finished'