function stats = modelStats(df, useValidSet, doBackTransform)
% function stats = modelStats(df, useValidSet, doBackTransform)
%
% Compute various statistics of the model and place them in a structure


error(nargchk(3,3,nargin));

[preds model condNumber] = trainModel(df, useValidSet, doBackTransform);

stats.condNumber = condNumber;

% process test set
if ~useValidSet 
    stats.runtimes = feval(df.backtransform, df.testdata.response, df);
    if isfield(df.testdata,'featureTimes') stats.featureTimes = df.testdata.featureTimes; end;
    if (doBackTransform)
		test = stats.runtimes;
	else
		test = df.testdata.response;
	end
else
    stats.runtimes = feval(df.backtransform, df.validdata.response, df);

    if isfield(df.validdata,'featureTimes') stats.featureTimes = df.validdata.featureTimes; end;
    if (doBackTransform)
		test = stats.runtimes;
	else
		test = df.validdata.response;
	end
end

% process training set
if (doBackTransform)
	train = feval(df.backtransform, df.traindata.response, df);
else
	train = df.traindata.response;
end

trainpreds = [df.traindata.features ones(size(df.traindata.features,1),1)] * model;
if (doBackTransform)
	trainpreds = feval(df.backtransform, trainpreds, df);
end

stats.MAE = mean(abs(preds-test));
stats.RMSE = sqrt(mean( (preds-test).^2 ));

stats.trainMAE = mean(abs(trainpreds - train));
stats.trainRMSE = sqrt(mean( (trainpreds - train).^2 ));

%NOTE: This needs doublechecking!
rss = sum( (trainpreds-train).^2 );
mss = sum( (train - mean(train)).^2 );

stats.r2 = 1 - rss / mss;

[n k]= size(df.traindata.features);
k = k+1; %account for intercept

stats.adjR2 = 1 - (1-stats.r2)* ((n-1)/(n-k));

stats.preds = preds;
stats.model = model;

%for the null hypothesis....
stats.testmean = mean(test);
