% function [model A Ainv XtY] = modelRemoveFeature(X, k, y, A, Ainv, delta, XtY)
% Train the linear model (X'X + I \delta)^{-1}X'y, given the matrix product
% and inverse from the previous iteration
function [model,A,Ainv,XtY] = modelSwapFeature(X, k, new_feature, y, A, Ainv, delta, XtY)

	% --------------------------
	% PART I - DELETION
	% --------------------------
	
	p = size(A,1);

	% swap rows and columns so that the row/column to be deleted is the last one
	temp = A(:,k);
	A(:,k:end-1) = A(:,k+1:end);
	A(:,end) = temp;
	temp = A(k,:);
	A(k:end-1,:) = A(k+1:end,:);
	A(end,:) = temp;
	
	% likewise, swap rows and columns in Ainv so that it is now the inverse
	% of the newly swapped version of A
	temp = Ainv(:,k);
	Ainv(:,k:end-1) = Ainv(:,k+1:end);
	Ainv(:,end) = temp;
	temp = Ainv(k,:);
	Ainv(k:end-1,:) = Ainv(k+1:end,:);
	Ainv(end,:) = temp;

	aBar = A(p,1:p-1); % row
	aDot = A(p,p);
	
	% first rank one update
	u1 = [-aBar'; -aDot+1];
	v1 = [zeros(p-1,1); 1];
	Ainv = Ainv - ((Ainv*u1)*(v1'*Ainv))/(1 + v1'*Ainv*u1);
	
	% second rank one update
	%u2 = v1;
	v2 = [-aBar'; 0];
	%Attinv = Atinv - ((Atinv*u2)*(v2'*Atinv))/(1 + v2'*Atinv*u2);
	Ainv = Ainv - (Ainv(:,p)*(v2'*Ainv))/(1 + v2'*Ainv(:,p));
	
	% I'm pretty sure the next line isn't needed, because of the "augment the matrix" line that follows in part II
	%Ainv = Ainv(1:p-1,1:p-1);
	
	% --------------------------
	% PART II - ADDITION 
	% --------------------------
	
%	tempX = X(:,1:end~=k)' * new_feature;  % this line is taking most of the runtime.
	tempX = X' * new_feature;
	tempX = tempX(1:end~=k,:);
%	tempX = X' * [new_feature(1:k-1); 0; new_feature(k:end)];
%	tempX = tempX(1:end~=k,:);
	A = [A(1:end-1,1:end-1) tempX; tempX' new_feature'*new_feature];  
	A(end,end) = A(end,end)+delta;

	% update the inverse stored in Ainv and train model
	p = size(A,1)-1;

	% augment the matrix (is there a better way?)
	Ainv(:,p+1) = zeros;
	Ainv(p+1,:) = zeros;
	Ainv(p+1,p+1) = 1;

	aBar = A(p+1,1:p); % row
	aDot = A(p+1,p+1);
	
	% first rank one update
	u1 = [zeros(p,1); 1];
	v1 = [aBar'; 0];
	Ainv = Ainv - ((Ainv*u1)*(v1'*Ainv))/(1 + v1'*Ainv*u1);
	
	% second rank one update
	u2 = [aBar'; aDot-1];
	v2 = u1;
	Ainv = Ainv - ((Ainv*u2)*(v2'*Ainv))/(1 + v2'*Ainv*u2);  % I've gotten division by zero runtime errors at this line
	
	% finally, build the model
	XtY = [XtY(1:end~=k,:); new_feature' * y];
	model = Ainv * XtY;

    