% function result = necessaryFeatures(df, subset, important_feature, threshold)
%
% variable importance sufficiency testing: for each "very important"
% feature in the small subset: (1) drop the feature from the subset; (2)
% consider adding every single feature which is not currently in the subset
% in turn; (3) compare the RMSE from each iteration of step 2 to the
% original RMSE of the whole subset.  Anything within some ratio (e.g.,
% 90%) gets considered a "good" substitute for the important feature.
function result = necessaryFeatures(df, subset, important_feature, threshold)

error(nargchk(4,4,nargin));

test = df.validdata.response;

features = subset.features(1:end~=find(subset.features==important_feature));
num_train_instances = size(df.traindata.features,1);
num_test_instances = size(df.validdata.features,1);
num_features = size(df.traindata.features,2);
t = size(features,2);

X = [ones(size(df.traindata.features,1),1) df.traindata.features(:,features)];
Xtest = [ones(num_test_instances,1) df.validdata.features(:,features) zeros(num_test_instances,1)];
delta = 1e-16;
A = X'*X+eye(t+1,t+1) *delta;
Ainv = inv(A);
XtY = X' * df.traindata.response;

% for each feature not in the set
result(1).RMSE = -1;
result(1).feature = -1;  % just make sure that something is always returned
index = 2;
for tt=1:size(df.traindata.features,2) %- size(features,2)
	if (~isempty(find(features==tt)) || tt == important_feature)
		continue;
	end

	% construct the model with this feature added
	model = modelAddFeature(X, df.traindata.features(:,tt), df.traindata.response, A, Ainv, delta, XtY);

	% record the RMSE and MAE on validation data for this feature
	Xtest(:,t+2) = df.validdata.features(:,tt);
	preds = Xtest*model;

	tempRMSE = sqrt(mean( (preds-test).^2 ));
	if (tempRMSE*threshold < subset.RMSE)
		result(index).RMSE = tempRMSE;
		result(index).feature = tt;
		index = index + 1;
	end
end
