function subs = subsetSelection(df, maxSize, method, useValidSet)
%function subs = subsetSelection(df, maxSize, type)
%
%Do subset selection on df, using method method, looking at subsets of max
%size. If method = 'all', then use the best subset found by all non-exhaustive methods.
%Returns a struct array of size 1..maxSize, containing selected features
%vector (colidx), rmse and mae of the model (on training data)

error(nargchk(4,4,nargin));

%choose a method
switch method
	case 'forwardSelection'
		subs=forwardSelection(df,maxSize);
	case 'was-all'
		subs=forwardSelection(df,maxSize);
	case 'backwardsSelection'
		%subs=backwardsSelection(df,maxSize);
        subs=backwardsSelection(df,1);
	case 'sequentialReplacement'
		subs=sequentialReplacement(df,maxSize);
	case 'all'
		display 'Subset Selection: running Forward, Backwards and Sequential'
		subs_forward = forwardSelection(df,maxSize);
		%subs_back = backwardsSelection(df,maxSize);
        subs_back = backwardsSelection(df,1);
		subs_sequential = sequentialReplacement(df,maxSize);
		for i=1:min(maxSize,size(df.traindata.features,2))
			bestRMSE = min(subs_forward(i).RMSE, min(subs_back(i).RMSE, subs_sequential(i).RMSE));
%			bestRMSE = min(subs_forward(i).RMSE, subs_back(i).RMSE);
			if (bestRMSE == subs_forward(i).RMSE) 
				subs(i) = subs_forward(i);
			elseif (bestRMSE == subs_back(i).RMSE)
				subs(i) = subs_back(i);
			else subs(i) = subs_sequential(i);
			end
		end
    case 'exhaustive'
        subs=exhaustiveSelection(df, maxSize);
        
	otherwise
		error ('invalid subset selection technique selected.');	
end

